from datetime import datetime
from pathlib import Path
from cruxpy.repo import port as cruxport


class page:
    """
    Class to handle website for portspage.
    It takes a path with ports, obtain their information.
    Then uses them with header and footer to prepare a page.

    """

    def __init__(self, path=".", header=""):

        self.ports_dir = Path(path, recursive=True)
        self.ports = sorted(list(self.ports_dir.glob("*/Pkgfile")))

        if not header:
            self.__make_header(Path(__file__).parent / "files/header.html")
        else:
            self.__make_header(Path(header))
        self.__make_table()
        self.__make_footer()

        self.content = "\n".join(self.header + self.table + self.footer)



    def __make_table(self):
        self.table = [
                '<table width="100%" cellspacing="0">',
                '<tr class="header"><td colspan="4">',
                '<strong>Signify public key:</strong> RWQXc2agtIcYBWpHqhhttRH+067uIV9sEkGPB2FT2uZRmFC712MDQF4Q</td></tr>',
                ]
        for index, port in enumerate(self.ports):
            current = "even" if index % 2 == 0 else "odd"
            p = cruxport(port)
            line = f'<tr class="{current}"><td><a href="{p.url}">{p.name}</a></td><td><a href="./{p.name}">{p.version}</a></td><td>{p.description}</td><td>{p.update}</td></tr>'
            self.table.append(line)
        self.table.append("</table>")


    def __make_header(self, file):

        with open(file) as f:
            content = f.read()
            self.header = content.splitlines()

    def __make_footer(self):
        self.footer = [
            f'<p class="footer">In total {len(self.ports)} ports',
            f'<p class="creator">Generated by portspage.py on {datetime.now()}</p>',
            "</body>",
            "</html>",
            ]


    def write(self,filepath='index.html'):
        """
        Write the page to the disk
        """
        with open(filepath,'w') as f:
            f.write(self.content)



import git
from datetime import datetime

class port:
    """
    Class for a Crux port information
    It takes the path to the Pkgfile and gather various informations,
    based of package build information and metdata.
    """

    def __init__(self, path):
        self.path= path
        self.pkgfile = str(path)
        self.__parse_pkgfile()
        self.name = self.fields["name"]
        self.description = self.__make_description()
        self.source = self.__make_source()
        self.url = self.__make_url()
        self.version = self.__make_version()

        self.update = self.last_update()

    def __parse_pkgfile(self):
        fields_tech = [
                "name",
                "release",
                "source",
                "version",
                ]
        fields_meta = [
                "depends_on",
                "description",
                "maintainer",
                "packager",
                "url",
                ]
        _fields_meta = ['# %s'%f.upper() if f == 'url' else '# %s'%f.capitalize() for f in fields_meta]

        with open(self.pkgfile) as f:
            self.pkgfile_raw = f.read()

        self.fields = {}
        lines = self.pkgfile_raw.strip().split('\n')
        for l in lines:
            if l.split("=")[0].strip() in fields_tech:
                words = l.split("=")
                self.fields[words[0].strip()] = words[1].strip()
            elif l.split(':')[0].strip() in _fields_meta:
                words = l.split(":")
                self.fields[words[0].strip('#').lower().strip()] = words[1:]


    def __make_description(self):
        return " ".join(self.fields["description"])

    def __make_source(self):
        s = self.fields["source"]
        s = s.strip("()")
        s = s.replace("$name",self.fields["name"])
        s = s.replace("$version",self.fields["version"])
        return s

    def __make_url(self):
        return "".join(self.fields["url"]).strip()

    def __make_version(self):
        return "%s-%s"%(self.fields["version"], self.fields["release"])

    def last_update(self):
       """Method to add the info about last update. It require git repo."""
        git_repo = git.Repo(self.path.parents[1])
        commit = list(git_repo.iter_commits(paths=self.path.parts[-2],max_count=1))[0]
        commit_date = datetime.fromtimestamp(commit.committed_date)
        return commit_date
